@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# Last changed Time-stamp: <2019-01-07 00:40:43 mtw>
+# Last changed Time-stamp: <2019-04-05 22:54:35 mtw>
 # -*-CPerl-*-
 #
 # usage: RNAalisplit.pl -a myfile.aln
@@ -7,7 +7,7 @@
 # NB: Display ID handling in Bio::AlignIO is broken for Stockholm
 # format. Use ClustalW format instead !!!
 
-use version; our $VERSION = qv('0.09');
+use version; our $VERSION = qv('0.10');
 use strict;
 use warnings;
 use Bio::RNA::RNAaliSplit;
@@ -22,14 +22,14 @@ use Path::Class;
 use File::Basename;
 use Carp;
 use RNA;
-#use diagnostics;
+use diagnostics;
 
 #^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^#
 #^^^^^^^^^^ Variables ^^^^^^^^^^^#
 #^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^#
 
 my $format = "ClustalW";
-my $method = "dHn"; # SCI | dHn | dHx | dBp | dHB
+my $method = "dHn"; # SCI | dHn | dHx | dBp | dBc | dHB
 my $rscape_stat = "GTp";
 my $outdir = "as";
 my $verbose = undef;
@@ -39,12 +39,26 @@ my ($alifile,$fi);
 my $scaleH = 1.;
 my $scaleB = 1.;
 my $ribosum = 1;
+my $constraint=undef;
 my $show_version = 0;
-my %foldme = (); # HoH of folded input sequences for dBp computation
+my %foldme = (); # HoH of folded input sequences for dBp/dBc computation
+
+my %pair = ("AU" => 5,
+	    "GC" => 1,
+	    "CG" => 2,
+	    "UA" => 6,
+	    "GU" => 3,
+	    "GT" => 3,
+	    "TG" => 4,
+	    "UG" => 4,
+	    "AT" => 5,
+	    "TA" => 6);
+
 
 Getopt::Long::config('no_ignore_case');
 pod2usage(-verbose => 1) unless GetOptions("aln|a=s"      => \$alifile,
-                                           "method|m=s"   => \$method,
+					   "constraint|c=s" => \$constraint,
+					   "method|m=s"   => \$method,
 					   "noribosum"    => sub{$ribosum=0},
 					   "out|o=s"      => \$outdir,
 					   "rscapestat=s" => \$rscape_stat,
@@ -52,9 +66,9 @@ pod2usage(-verbose => 1) unless GetOptions("aln|a=s"      => \$alifile,
 					   "scaleB"       => \$scaleB,
 					   "verbose|v"    => sub{$verbose = 1},
 					   "version"      => sub{$show_version = 1},
-                                           "man"          => sub{pod2usage(-verbose => 2)},
-                                           "help|h"       => sub{pod2usage(1)}
-                                           );
+					   "man"          => sub{pod2usage(-verbose => 2)},
+					   "help|h"       => sub{pod2usage(1)}
+					   );
 
 if ($show_version == 1){
   print "RNAalisplit $VERSION\n";
@@ -66,9 +80,16 @@ unless (-f $alifile){
   pod2usage(-verbose => 0);
 }
 
+croak "ERROR: method dBc must be selected when using constraints, exiting ..."
+  if (defined $constraint && $method ne "dBc");
+
 if($method eq "dBp" || $method eq "dHB"){
   $fi = fold_input_alignment($alifile); # for later computation of BP dist
 }
+if($method eq "dBc"){
+  $fi = fold_input_alignment_constrained($alifile,$constraint);
+}
+
 
 my $round = 1;
 my $done = 0;
@@ -79,8 +100,8 @@ while ($done != 1){
   print STDERR "Computing round $lround ...\n";
   alisplit($alifile,$odirname);
   $round++;
-  $done = 1; 
-  #TODO sort output by RNAz SVM prob and re-run with the first few as input alignments 
+  $done = 1;
+  #TODO sort output by RNAz SVM prob and re-run with the first few as input alignments
 }
 
 
@@ -234,14 +255,21 @@ sub make_distance_matrix {
     elsif ($m eq "dHx") { # hamming dist with all gap columns removed
       $D[$dim*($i-1)+($j-1)] =  $D[$dim*($j-1)+($i-1)] = $dHx;
     }
-    elsif ($m eq "dBp") { # base pairdistance
+    elsif ($m eq "dBp") { # base pair distance
+      carp "ERROR [$this_function] sequence \'display_id\' not found in input alignment"
+	unless (exists $$fi{$id1} && exists $$fi{$id2});
+      my $gss1 = $$fi{$id1}->{gss};
+      my $gss2 = $$fi{$id2}->{gss};
+      my $dBp = RNA::bp_distance($gss1,$gss2);
+      $D[$dim*($i-1)+($j-1)] =  $D[$dim*($j-1)+($i-1)] = $dBp;
+    }
+    elsif ($m eq "dBc") { # constrained base pair distance
       carp "ERROR [$this_function] sequence \'display_id\' not found in input alignment"
 	unless (exists $$fi{$id1} && exists $$fi{$id2});
       my $gss1 = $$fi{$id1}->{gss};
       my $gss2 = $$fi{$id2}->{gss};
       my $dBp = RNA::bp_distance($gss1,$gss2);
       $D[$dim*($i-1)+($j-1)] =  $D[$dim*($j-1)+($i-1)] = $dBp;
-
     }
     elsif($m eq "dHB") { # combined hamming + basepair distance
       carp "ERROR [$this_function] sequence \'display_id\' not found in input alignment"
@@ -252,7 +280,7 @@ sub make_distance_matrix {
       my $dHB = $scaleH*$dHn + $scaleB*$dBp;
       $D[$dim*($i-1)+($j-1)] =  $D[$dim*($j-1)+($i-1)] = $dHB;
     }
-    else {croak "Method $method not available ..please use SCI|dHn|dHx|dBp|dHB"}
+    else {croak "Method $method not available ..please use any of SCI|dHn|dHx|dBp|dBc|dHB"}
   }
 
   # write matrix to file
@@ -261,6 +289,7 @@ sub make_distance_matrix {
   elsif ($m eq "dHn"){$Dfile = dump_matrix(\@D,$dim,1,1,"dHn",$ASO,$verbose)}
   elsif ($m eq "dHx"){$Dfile = dump_matrix(\@D,$dim,1,1,"dHx",$ASO,$verbose)}
   elsif ($m eq "dBp"){$Dfile = dump_matrix(\@D,$dim,1,1,"dBp",$ASO,$verbose)}
+  elsif ($m eq "dBc"){$Dfile = dump_matrix(\@D,$dim,1,1,"dBc",$ASO,$verbose)}
   elsif ($m eq "dHB"){$Dfile = dump_matrix(\@D,$dim,1,1,"dHB",$ASO,$verbose)}
   else { croak "Method $m not available ..please use SCI|dHn|dHx|dBp|dHB"}
 
@@ -286,6 +315,7 @@ sub dump_matrix {
   elsif($what eq "dHn"){$info="> H (Hamming distance with gap Ns)"}
   elsif($what eq "dHx"){$info="> H (Hamming distance with gaps removed)"}
   elsif($what eq "dBp"){$info="> B (Base pair distance)"}
+  elsif($what eq "dBc"){$info="> B (Base pair distance with constraints)"}
   else{$info="> H (unknown)"}
 
   if (defined $ad){ # print lower triangle matrix input for AanalyseDists
@@ -358,7 +388,7 @@ sub fold_input_alignment {
     my @gappos = ();
     my $gseq = $element->seq;
     for (my $i=0;$i<length($gseq);$i++){
-      push @gappos, $i if (substr($gseq,$i,1) eq "-"); # keep gap positions
+      push @gappos, $i if (substr($gseq,$i,1) eq "-"); # store gap positions
     }
     my $seq = $gseq;
     $seq =~ s/-//g; # get gap-free sequence
@@ -369,8 +399,8 @@ sub fold_input_alignment {
       substr($gss,$gappos[$i]+$gaps_inserted,0) = ".";
       $gaps_inserted++;
     }
-   # print $element->display_id."\n$gseq\n$gss\n$seq\n$ss\n";
-   # print join ",", @gappos,"\n---\n";
+    print $element->display_id."\n$gseq\n$gss\n$seq\n$ss\n";
+    print join ",", @gappos,"\n---\n";
     unless (exists($foldin{$element->display_id})){
       $foldin{$element->display_id} = {
 				       gseq => $gseq,
@@ -387,6 +417,105 @@ sub fold_input_alignment {
   return \%foldin;
 }
 
+sub fold_input_alignment_constrained {
+  my ($aln,$constr) = @_;
+  my %foldin = ();
+  my @pt = make_pair_table($constr);
+  my $input_AlignIO = Bio::AlignIO->new(-file => $alifile,
+				      -format => 'ClustalW'
+				     );
+  my $input_aln = $input_AlignIO->next_aln;
+  foreach my $element ($input_aln->each_seq) {
+    my @gappos = ();
+    my $cons=$constr;
+    my $gseq = $element->seq;
+    my $id = $element->display_id;
+    if (length($constraint) != length($gseq)){
+      print STDERR "ERROR: constraint length does not match alignment length ...\n$constraint \n $gseq";
+      croak();
+    }
+    print STDERR "\n>$id\n";
+    print STDERR "$gseq\n";
+    for (my $p=0; $p<length($gseq);$p++) {
+      # remove non-compatible pairs as well as pairs to a gap position
+      my $c = substr($gseq,$p,1);
+      if ($c eq '-') {
+	push @gappos, $p;
+	substr($cons,$p,1) = 'x'; # mark for removal
+	substr($cons,$pt[$p],1) = '.'
+	  if $pt[$p]>0  && substr($cons,$pt[$p],1) ne 'x';   # open pair
+      }
+      elsif ($pt[$p]>$p) {
+	substr($cons,$p,1) = substr($cons,$pt[$p],1) = '.'
+	  unless exists $pair{$c . substr($gseq,$pt[$p],1)};
+      }
+    } # end for
+
+    print STDERR "$cons\n$gseq\n";
+    # print STDERR length($seq), length($cons), "\n";
+    $cons =~ s/x//g;
+    my $seq = $gseq;
+    $seq  =~ s/-//g;
+    print STDERR "$cons\n$seq\n";
+
+    # do constraint folding
+    $RNA::fold_constrained = 1;
+    my ($css,$cmfe) =  RNA::fold($seq, $cons);
+    print STDERR "$css\t$cmfe\n";
+    $RNA::fold_constrained = 0;
+
+    # re-insert gaps into constraint-folded strcuture
+    my $gcss = $css;
+    for (my $i=0;$i<=$#gappos;$i++){
+      my $gaps_inserted = 0;
+      substr($gcss,$gappos[$i]+$gaps_inserted,0) = ".";
+      $gaps_inserted++;
+    }
+    print "$gcss\n";
+    print STDERR join ",", @gappos,"\n";
+
+    unless (exists($foldin{$element->display_id})){
+      $foldin{$element->display_id} = {
+				       gseq => $gseq,
+				       seq => $seq,
+				       ss => $css,
+				       gss =>$gcss,
+				       mfe => $cmfe,
+				      }
+    }
+    else{
+      croak "ERROR: duplicate sequence identifier in input: ".$element->display_id;
+    }
+    print STDERR Dumper( $foldin{$element->display_id} );
+    print STDERR "---\n";
+  }
+  return \%foldin;
+}
+
+sub make_pair_table {
+   #indices start at 0 in this version!
+  my $structure = shift;
+  print ">>$structure<<\n";
+   my (@olist, @table);
+   my ($hx,$i) = (0,0);
+
+   foreach my $c (split(//,$structure)) {
+       if ($c eq '.') {
+	  $table[$i]= -1;
+     } elsif ($c eq '(') {
+	  $olist[$hx++]=$i;
+     } elsif ($c eq ')') {
+	 my $j = $olist[--$hx];
+	 die ("unbalanced brackets in make_pair_table") if ($hx<0);
+	 $table[$i]=$j;
+	 $table[$j]=$i;
+     }
+      $i++;
+   }
+   carp ("too few closed brackets in make_pair_table") if ($hx!=0);
+   return @table;
+}
+
 
 __END__
 
@@ -427,18 +556,27 @@ from pairwise distances. It can be visualized e.g. with SplitsTree.
 
 A multiple sequence alignment in ClustalW format
 
+=item B<--constraint|-c>
+
+Constraint structure, overriding the consensus structure of the
+underlying alignment in case  B<--method|-m dBc> is selected.
+
 =item B<--method|-m>
 
 Method to compute pairwise ditances. Available options are 'dHn',
-'dHx', 'dBp', and 'SCI'. The first and second compute pairwise Hamming
-distances of sequences, where 'dHn' replaces gaps with 'N', whereas
-'dHx' removes all gap columns (is not yet implemented). 'dBp' folds
-RNA sequences into thir MFE structures and computes pairwise base pair
-distances. 'SCI' computes the distance as 1-log(SCI), based on a
-truncated strucure conservation index of two sequences. The latter,
-however, is not a metric and therefore often results in negative
-branch lengths in Neighbor Joining trees. Use with caution. [default:
-'dHn']
+'dHx', 'dBp', 'dBc, 'dHB', and 'SCI'. The first and second compute
+pairwise Hamming distances of sequences, where 'dHn' replaces gaps
+with 'N', whereas 'dHx' removes all gap columns (not yet
+implemented). 'dBp' folds RNA sequences into their MFE structures and
+computes pairwise base pair distances. 'dBc' computes base pair
+distances on constraint-folded RNA sequences. Here, the default is to
+use the consensus structure of the underlying alignment as a
+constraint, however, an alternative constraint structure can be
+provided via the B<--constraint> option. 'SCI' computes the distance
+as 1-log(SCI), based on a truncated strucure conservation index of two
+sequences. The latter, however, is not a metric and therefore often
+results in negative branch lengths in Neighbor Joining trees. Use with
+caution. [default: 'dHn']
 
 =item B<--noribosum>
 
